#!/bin/bash
# Jethro environment script
# 12.8.2014 added bash auto-complete 
# 4.3.2014 first version


###################################
# Configure JETHRO_HOME environment
###################################

if [ -z "${JETHRO_HOME}" ]
then
    JETHRO_HOME=/opt/jethro/current
    export JETHRO_HOME
fi

export JETHRO_USER=`ls -ld ${JETHRO_HOME}/bin/Jethro | awk '{print $3}'`
export JETHRO_GROUP=`ls -ld ${JETHRO_HOME}/bin/Jethro | awk '{print $4}'`

###################
# Configure aliases
###################
unalias JethroClient 2>/dev/null       #remove rlwrap based wrapper of JethroClient
alias jethroclient="JethroClient"
alias JQ="rlwrap Jethro Q"

##############################
# Configure bash auto-complete
#############################

_Jethro() 
{
    # auto-complete Jethro instance name
    local cur prev opts
    COMPREPLY=()
    cur="${COMP_WORDS[COMP_CWORD]}"

    # Jethro executable requires instance name as the second parameter
    if [[ ${COMP_WORDS[0]} == "Jethro" ]]; then
        if [[ $COMP_CWORD -eq 2 ]]; then
            instances_full_path=`ls -d $JETHRO_HOME/instances/*/ 2> /dev/null`
            instances=`for i in $instances_full_path ; do basename $i ; done`
            COMPREPLY=( $(compgen -W "${instances}" -- ${cur}) )
            return 0
        fi
    else 
        # Other JethroData executables require instance name as the first parameter
        if [[ $COMP_CWORD -eq 1 ]]; then
            instances_full_path=`ls -d $JETHRO_HOME/instances/*/ 2> /dev/null` 
            instances=`for i in $instances_full_path ; do basename $i ; done`
            COMPREPLY=( $(compgen -W "${instances}" -- ${cur}) )
            return 0
       fi
   fi
   # For all other paramters, do file-based autocomplete
   COMPREPLY=( $(compgen -f -- ${cur}) )
   return 0
}

_JethroAdmin()
{
    local cur prev opts
    COMPREPLY=()
    cur="${COMP_WORDS[COMP_CWORD]}"
    prev="${COMP_WORDS[COMP_CWORD-1]}"
    opts="list-instances list-storage-instances create-instance delete-instance attach-instance detach-instance change-pass collect-logs"
    # For the first parameter, complete from the list
    if [[ $COMP_CWORD -eq 1 ]]; then
        COMPREPLY=($(compgen -W "${opts}" -- ${cur}))
        return 0
    fi
    # for the second paramter, complete using instance name if expecting one
    if [[ $COMP_CWORD -eq 2 ]]; then
        if [[ ${prev} == "delete-instance" || ${prev} == "detach-instance" || ${prev} == "reset-instance" || ${prev} == "collect-logs" || ${prev} == "change-pass" ]]; then
            instances_full_path=`ls -d $JETHRO_HOME/instances/*/ 2> /dev/null`
            instances=`for i in $instances_full_path ; do basename $i ; done`
            COMPREPLY=( $(compgen -W "${instances}" -- ${cur}) )
        fi
        return 0
   fi
   # For all other paramters, do file-based autocomplete 
   COMPREPLY=( $(compgen -f -- ${cur}) )
   return 0
}

type complete >/dev/null 2>&1
if [[ $? -eq 0 ]]; then 
    complete -F _JethroAdmin JethroAdmin
    complete -F _Jethro JethroServer
    complete -F _Jethro JethroClient
    complete -F _Jethro JethroLoader
    complete -F _Jethro JethroMaint
    complete -F _Jethro JethroLoadsScheduler
    complete -F _Jethro Jethro
fi

