#!/bin/bash
#
#Renew Jethro Kerberos Ticket
#Parmas configured by JethroAdmin set.host.config -D param=value [-D param=value]
#
#hdfs_kerberos_keytab - keytab location
#hdfs_kerberos_principal - principal name
#hdfs_kerberos_reinit_interval_sec - check interval for renew or regenerate
#status1 - first check by jethro monitor, checks host-conf file and configuration validation
#status2 - second check verify kinit works
#status3 - loop run for full follow

#If status is empty use status3 - main fllow
monitorStatus=$1
if [ "X$monitorStatus" == "X" ]; then
	monitorStatus=status3
fi
#Global parameters
hostconf=/opt/jethro/config/host-conf.ini
logfile=/var/log/jethro/monitor.log
#Check interval(in seconds) in case the configuration was changes during status3 
CHECK_INTERVAL=30
export JETHRO_USER=`ls -ld ${JETHRO_HOME}/bin/Jethro | awk '{print $3}'`

#get config parameters, takes the parameters, ignores "#", takes the last value after "=", remove all spaces
function check_hdfs_kerberos_keytab {
	(awk '/^'hdfs'\.'kerberos'\.'keytab'/' $hostconf | awk -F "=" '/^[^#]/ { print $2 }' | tail -1 | sed 's/\ //g' )
}
function check_hdfs_kerberos_principal {
	(awk '/^'hdfs'\.'kerberos'\.'principal'/' $hostconf | awk -F "=" '/^[^#]/ { print $2 }' | tail -1 | sed 's/\ //g' )
}
function check_hdfs_kerberos_reinit_interval_sec {
	(awk '/^'hdfs'\.'kerberos'\.'reinit'\.'interval'\.'sec'/' $hostconf | awk -F "=" '/^[^#]/ { print $2 }' | tail -1 | sed 's/\ //g' )
}
function getconfvaribals {
	hdfs_kerberos_keytab=$(check_hdfs_kerberos_keytab)
	hdfs_kerberos_principal=$(check_hdfs_kerberos_principal)
	hdfs_kerberos_reinit_interval_sec=$(check_hdfs_kerberos_reinit_interval_sec)
	if [ -z $hdfs_kerberos_reinit_interval_sec ] ; then
		#if null default value is 1 hour
		hdfs_kerberos_reinit_interval_sec=3600
	fi
}
#Global functions 
function logprint { 
	echo $(date +%Y-%m-%d" "%H:%M:%S) kerberosTicketRenewal:
}
function confmodification { 
	stat -c %Y $hostconf
}
#check host-conf modification date
hostConfModified=$(confmodification)

#Initialize config parameters
getconfvaribals

#Kerberos Initialize function, if fails alert the JethroMonitor on status2 only otherwise print to log
function generateNewTicktes {
echo  "$(logprint) Generating Kerberos ticket for $hdfs_kerberos_principal, keytab:$hdfs_kerberos_keytab" >> $logfile
kinit -kt $hdfs_kerberos_keytab $hdfs_kerberos_principal &> /dev/null
if [ $? -ne 0 ]; then
        echo "$(logprint) ERROR: An error occurred while generating ticket, please check $hostconf" >> $logfile
        if [ $monitorStatus = status2 ] ; then
				echo "$(logprint) ===================== Stop kerberosTicketRenewal ========================" >> $logfile
                exit 21
        fi
else
        echo "$(logprint) Ticket was generated successfully for $hdfs_kerberos_principal keytab:$hdfs_kerberos_keytab" >> $logfile
        if [ $monitorStatus = status2 ] ; then
				echo "$(logprint) ===================== Stop kerberosTicketRenewal ========================" >> $logfile
                exit 20
        fi
fi
}

echo "$(logprint) ===================== Start kerberosTicketRenewal ========================" >> $logfile
#Status1 checks that the host-conf.ini file and all the configurations for Kerberos are present 
if [ $monitorStatus = status1 ] ; then
	#Print first check status
	#If config file does not exist, exit.
	if [ -s "$hostconfig" ]; then
		echo "$(logprint) ERROR: Kerberos configuration file $hostconf is missing, Kerberos renewal ticket process is disabled" >> $logfile
		echo "$(logprint) ===================== Stop kerberosTicketRenewal ========================" >> $logfile
		exit 11
	fi
	#Checking mandatory parameters, if not exist, exit.
    if [ -z $hdfs_kerberos_keytab ] || [ -z $hdfs_kerberos_principal ] || [ -z $hdfs_kerberos_reinit_interval_sec ]; then
		echo "$(logprint) Kerberos parameters are missing in $hostconf, Kerberos was not enabled" >> $logfile
		echo "$(logprint) Mandatory parameters are hdfs.kerberos.keytab, hdfs.kerberos.principal" >> $logfile
		echo "$(logprint) ===================== Stop kerberosTicketRenewal ========================" >> $logfile
        exit 11
    else
		echo "$(logprint) ===================== Stop kerberosTicketRenewal ========================" >> $logfile
        exit 10
    fi
fi
#Status2 check, validate that kinit works,if true notifies JethroMonotor to start status3 otherwise alters sends an alert to stop  
if [ $monitorStatus = status2 ] ; then
    generateNewTicktes
fi

#Checks if host-conf was changed
function check_hostConfModification {
	hostConfModified2=$(confmodification)
	if [ $hostConfModified -ne $hostConfModified2 ] ; then
		getconfvaribals
		hostConfModified=$hostConfModified2
	fi
}

#Main loop starts here
while true ; do	
	#Checks if host-conf was changed on every loop to keep dynamic update 
	check_hostConfModification

	#Check config, sleep if at least one of the parameter is empty, set to 120 to get the log error for the first time before entering the loop
	countloop=120
    while [ -z $hdfs_kerberos_keytab ] || [ -z $hdfs_kerberos_principal ] || [ -z $hdfs_kerberos_reinit_interval_sec ]; do
        #checks every 120 loops (1 hour), logs an error if the configuration was not set
        if (( $countloop % 120 == 0 )); then
			countloop=0
            echo  "$(logprint) ERROR: Kerberos parameters are missing in $hostconf, please use JethroAdmin set.host.config" >> $logfile
        fi
		(( countloop++ ))
		sleep $CHECK_INTERVAL
		check_hostConfModification
	done	
	
	#Log params
	echo "$(logprint) Found initial parameters hdfs.kerberos.keytab:$hdfs_kerberos_keytab, hdfs.kerberos.principal:$hdfs_kerberos_principal, hdfs.kerberos.reinit.interval.sec:$hdfs_kerberos_reinit_interval_sec" >> $logfile
	
	
	#MAIN SCRIPT
	#Checks if the ticket is valid
	klist -s
	
	#If no ticktes found at all - create them
	if [[ ! $? -eq 0 ]]; then
		echo "$(logprint) Ticket was not found, generating new ticket for $hdfs_kerberos_principal, keytab:$hdfs_kerberos_keytab" >> $logfile
		generateNewTicktes
	else
		#Find cache file
		for CACHE_FILE in $(find /tmp -maxdepth 1 -type f -name "krb5cc*$UID"); do
			echo "$(logprint) Ticket cache found for $JETHRO_USER at: $CACHE_FILE" >> $logfile
			EXPIRE_TIME=$( date -d "$( klist -c $CACHE_FILE | grep krbtgt | awk '{print $3, $4}' )" +%s )
			echo "$(logprint) Ticket expiration date: $(date -d @$EXPIRE_TIME)" >> $logfile
				#If it has already expired, delete it
				if [ $( date +%s ) -ge $EXPIRE_TIME ]; then
					#delete cache file
					kdestroy -c $CACHE_FILE &> /dev/null
					echo "$(logprint) Ticket cache ($CACHE_FILE) for user $JETHRO_USER is expired" >> $logfile
					#regenerate ticket
					generateNewTicktes
		
				#Otherwise renew it
				elif [ $( expr $EXPIRE_TIME - $( date +%s ) ) -le $(($hdfs_kerberos_reinit_interval_sec + 300)) ]; then
					echo "$(logprint) The ticket is about to expire" >> $logfile
					echo "$(logprint) Renewing ticket for $JETHRO_USER using $CACHE_FILE" >> $logfile
					kinit -R -c $CACHE_FILE &> /dev/null
					#If renew fails for any reason try to regenerate 
					if [ ! $? -eq 0 ]; then
						echo "$(logprint) Ticket renew failed for user:$JETHRO_USER $CACHE_FILE" >> $logfile
						echo "$(logprint) Trying to regenerate the ticket using $hdfs_kerberos_principal, keytab:$hdfs_kerberos_keytab" >> $logfile
						generateNewTicktes
					else
						echo "$(logprint) Ticket was renewed Successfully" >> $logfile
					fi
				else
					#If all good and ticker is still valid until the next check interval + 5 minutes, do nothing
					echo "$(logprint) Ticket is valid for $JETHRO_USER, next check in $hdfs_kerberos_reinit_interval_sec seconds" >> $logfile
				fi
		done
	fi
		#Sleep until the next check interval (1 hour by default)
		sleep $hdfs_kerberos_reinit_interval_sec
done
