#!/bin/bash

MONITOR_PATH=$JETHRO_HOME/bin
PID_PATH=/var/run/jethro

servicesmonitor_pid=$(cat $PID_PATH/servicesmonitor.pid 2>&1)
echo -e "$(date +%Y-%m-%d" "%H:%M:%S) servicesmonitor.sh: servicesmonitor is up and running PID=$servicesmonitor_pid"  >> /var/log/jethro/monitor.log 2>&1

# infinity loop for monitoring the servicesmonitor service
# this script will start the servicesmonitor service in case that the service is down
while :
do
	# JETHRO_HOME init
	. /etc/profile.d/jethro.sh
	
	#
	
	SLEEP=8
	STOP=5
	NOW=`date +"%s"`
	
	# set to 0 to disable ping responsiveness check
	PING_TIMEOUT_MILLISEC=15000
	
	
	LOCKFILE=/var/run/jethro/monitor.lock
	
	function touch_status {
		if [[ -s /tmp/jethro${1}.st ]]; then
		if [[ $(((NOW-`head -1 /tmp/jethro${1}.st`)/60)) -gt 4 ]]; then
			cat /dev/null > /tmp/jethro$1.st
		fi
		else
		touch /tmp/jethro$1.st >>/var/log/jethro/monitor.log 2>&1
		fi
	}
	
	# (in:pid)
	function get_port {
			local l_in_pid=$1
			local l_port
		if [[ `ps --no-headers --pid "$l_in_pid"` ]]; then
		l_port=`ps -f --no-headers --pid "$l_in_pid" | awk {'print($10)'}`
			fi
			echo $l_port
	}
	
	function restart_server {
		local l_in_instance=$1
		local l_in_service=$2
	
		#create tmp file to preserve PID of finished service
		pid_temp_file=$(mktemp)
		# sed prevents control sequences from being written into logs
		$JETHRO_HOME/bin/jethro.initd stop $l_in_instance ${l_in_service} 3>$pid_temp_file | sed -E 's/\x1b\[[0-9;]*m?//g' >>/var/log/jethro/monitor.log 2>&1
	
		PID=`cat "$pid_temp_file"`
		rm -f $pid_temp_file
		# ensure service is down -- otherwise sleep and try again
		if [ -n $PID ]; then
			while ( kill -0 "$PID" 2>/dev/null ); do
				sleep 0.5
			done
		fi
	
		# service is finally down -- start it anew
		echo -en "$(date +%Y-%m-%d" "%H:%M:%S) servicesmonitor.sh: "  >> /var/log/jethro/monitor.log 2>&1
		$JETHRO_HOME/bin/jethro.initd start $l_in_instance ${l_in_service} | sed -E 's/\x1b\[[0-9;]*m?//g' >>/var/log/jethro/monitor.log 2>&1
	}
	
	# (in:instance, in:pid)
	function do_ping {
	local l_in_instance=$1
	local l_in_pid=$2
	local l_port
	l_port=$(get_port $l_in_pid)
	
	$JETHRO_HOME/bin/JethroClient $l_in_instance localhost:$l_port --ping $PING_TIMEOUT_MILLISEC &> /dev/null
	local l_ping_res=$?
	if [[ $l_ping_res -gt 1 ]]; then
			echo "-- `date +"%Y %m %d %H:%M:%S"`: instance $l_in_instance failed ping. Restarting!" >>/var/log/jethro/monitor.log 2>&1
			echo "`date +"%Y-%m-%d %H:%M:%S"` ERROR ************************* $0: Instance $l_in_instance failed ping (code=${l_ping_res}). Restarting! (this message is logged from the monitor script)" >>/var/log/jethro/${l_in_instance}/jethro.log 2>&1
			restart_server $l_in_instance
	fi
	
	}
	
	function check {
	if [[ `ls /var/run/jethro/ | grep jethro${1}* | wc -l` -gt 0 ]]; then
		for PIDF in /var/run/jethro/jethro${1}*
		do
			PID_INS=`basename $PIDF`
			PID=`cat /var/run/jethro/$PID_INS`
			instance_temp=${PID_INS#*_}            # remove `jethro..._' prefix
			instancename=${instance_temp%.*}       # remove file extension suffix
	
			if [[ ! `ps -ed | grep "$PID"` ]]; then
				if [[ `cat /tmp/jethro.st | wc -l` -gt $STOP ]]; then
				echo "-- `date +"%Y %m %d %H:%M:%S"`"
				echo "$STOP crashes in 5 minutes on $1, stopping recovery of the $instancename."
				rm -rf /var/run/jethro/$PID_INS >>/var/log/jethro/monitor.log 2>&1
				break
				else
				echo "-- `date +"%Y %m %d %H:%M:%S"`"
				echo "This PID - $PID doesn't look running. Restarting the jethro $1 '$instancename'"
				rm -rf /var/run/jethro/$PID_INS >>/var/log/jethro/monitor.log 2>&1
				SERVICE=${1#server}
							restart_server $instancename $SERVICE
				echo "`date +"%s"`" >> /tmp/jethro$1.st
				break
				fi
					else
					if [[ "$1" = "server" ]]; then
							local l_cycle_number=$2
						#server is up lets check its not stuck (only on last cycle)
							if [[ $l_cycle_number -eq 6 && $PING_TIMEOUT_MILLISEC -ne 0 ]]; then
									do_ping $instancename $PID
							fi
					fi
			fi
		done
		fi
	}
	
	# make sure only single instance (cron may have started another one before prev finished)
	#[ -e ${LOCKFILE} ] && active_monitor_pid=`cat ${LOCKFILE}`;
	#if [ ! -z ${active_monitor_pid} ] && kill -0 $active_monitor_pid; then
	#	echo "`date`: Another monitor already running with pid $active_monitor_pid. Terminating." >>/var/log/jethro/monitor.log 2>&1
	#	exit
	#fi
	
	# make sure the lockfile is removed when we exit and then claim it
	trap "rm -f ${LOCKFILE}; exit" INT TERM EXIT
	echo $$ > ${LOCKFILE}
	
	touch_status server
	touch_status maint
	touch_status loadscheduler
	
	for i in {1..6}; do
	check server $i
	check maint
	check loadscheduler
	sleep $SLEEP
	done
	
	rm -f ${LOCKFILE} &> /dev/null

	# check if JethroMonitor service is up and running, if not, then start it
	JethroMonitor_pid=$(cat $PID_PATH/JethroMonitor.pid 2>&1)
	kill -0 $JethroMonitor_pid > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		# Process is running
		:
	else
		$MONITOR_PATH/JethroMonitor 2>&1 & echo $! > $PID_PATH/JethroMonitor.pid &
		JethroMonitor_pid=$(cat $PID_PATH/JethroMonitor.pid 2>&1)
		echo -e "$(date +%Y-%m-%d" "%H:%M:%S) servicesmonitor.sh: JethroMonitor service was not found. Restarting... PID=$JethroMonitor_pid"  >> /var/log/jethro/monitor.log 2>&1
	fi
done
