function display_usage {
echo "Upgrade to 3.4 - a utility to upgrade Jethro instance to work with multi-schema"
echo "Before running, please run 'service jethro stop <instance_name> all' on ALL instance hosts and make sure no loads are active."
echo "USAGE: upgradeTo3.4_MultiSchema instance_name connection-string -u username -p password "
}

function connect_to_server {
connected=0
count=0
done=0
while [ $count -lt 30 ]
do
 echo  "Connecting to JethroServer..."
 JethroClient $INSTANCE $CONNECTION -u $USER -p $PASSWORD  -q "show version" >/dev/null 2>&1 
 if [ $? == 0  ]
 then
 echo  "Connected successfuly to JethroServer."
 break
 else
	if [[ "$(JethroClient $INSTANCE $CONNECTION -u $USER -p $PASSWORD  -q "show version" |  grep "Connection rejected - Authentication failed, check username and password." | wc -l)" -ne 0 ]]; then
	JethroClient $INSTANCE $CONNECTION -u $USER -p $PASSWORD  -q "show version"  #print to the user the error message for the user\password
	echo "Failed to connect, exiting..."
	exit 1
	fi
 fi
 echo "Couldn't connect to Jethro Server, will try again in 30 sec"
 sleep 30
 count=$[$count+1]
done

 if [ $count -lt 30  ]
 then
 connected=1
 fi
}

function create_new_table {
        JethroClient $INSTANCE $CONNECTION -u $USER -p $PASSWORD -i $JETHRO_HOME/bin/system_tables.ddl >/dev/null #creates table and scheduled load
}

function move_old_table {
        echo  "Dropping old scheduled load for table jethro\$_loader_events..."
        Jethro U $INSTANCE X SCHEDULE 'def_schema.jethro$_loader_events'
        if [[ "$?" -ne 0 ]]; then
    echo "Failed to drop scheduled load."
        fi

    echo "Stopping Jethro Server"
        service jethro stop  $INSTANCE

        echo  "Moving jethro\$_loader_events to sys schema..."
        Jethro U $INSTANCE Y "def_schema.jethro\$_loader_events" "sys.jethro\$_loader_events"
    if [[ "$?" -ne 0 ]]; then
    echo "Failed to move system table. system table is still under def_schema."
        fi

        echo "Starting Jethro Server"
        service jethro start  $INSTANCE

connect_to_server

 if [ $connected == 0  ]
 then
        echo "Failed to start server, something went wrong"
        display_usage
        exit 1
fi

        echo  "Adding new column 'targetschema' to table jethro\$_loader_events..."
        JethroClient $INSTANCE $CONNECTION -u $USER -p $PASSWORD  -q "ALTER TABLE sys.jethro\$_loader_events ADD COLUMN targetschema STRING;"
        if [[ "$?" -ne 0 ]]; then
    echo "Failed to add new column."
        fi

         echo  "Creating new schedule load for sys table"
     JethroClient $INSTANCE $CONNECTION -u $USER -p $PASSWORD  -q "CREATE SCHEDULED LOAD sys.jethro\$_loader_events from jethro\$_loader_events*.dat SCHEDULE CONTINUES POSTLOAD_FILE_ACTION delete SYSTEM_TABLE;"
     if [[ "$?" -ne 0 ]]; then
     echo "Failed to create new Scheduled load."
	fi

}
###############################
# MAIN SCRIPT STARTS HERE
###############################
ARGS=( "$@" )
# General checks
if [[ $EUID -eq 0 ]]; then
  # Running as root messes local file permissions
  echo
  echo 'Error - this script must run as a non-root user, exiting'
  exit 1
fi

if [[ $# -lt 7 ]]; then
  INSTANCE=`echo $1 | tr [:upper:] [:lower:]`
  CONNECTION=`echo $2`
  if [[ "$3" != "-u" ]] || [[ "$5" != "-p" ]]; then
                echo -e "Error, invalid input."
                display_usage
                exit 1
        fi
        USER=`echo $4`
        PASSWORD=`echo $6`
else
  display_usage
  exit 1
fi

echo -e "\e[31m Important - Before starting, please run 'service jethro stop $INSTANCE all' on ALL instance hosts, and make sure no loads are active.  \e[0m" 
read -r -p "Continue? [Y/N]" yn 
  if [[ $yn =~ ^([yY][eE][sS]|[yY])$ ]]; then
		#nothing, will continue
		echo;
	else
       echo "Canceled, exiting..."; 
	   	exit 0;
   fi
   
echo "Stopping Jethro LoadScheduler and maint"
service jethro stop  $INSTANCE loadscheduler
service jethro stop  $INSTANCE maint

echo "Starting Jethro Server"
service jethro start  $INSTANCE 


	   
connect_to_server
 if [ $connected == 0  ]
 then
        echo "Failed to connect. please check all cradentials are correct"
        echo
        display_usage
        exit 1
fi


echo  "Creating sys schema..."
JethroClient $INSTANCE $CONNECTION -u $USER -p $PASSWORD  -q "create schema sys;"
if [[ "$?" -ne 0 ]]; then
	echo "Failed to create sys schema."
fi
		
if [[ "$(JethroClient $INSTANCE $CONNECTION  -u $USER -p $PASSWORD  -q "select * from def_schema.jethro\$_loader_events limit 0;" 2>&1 |  grep "does not exist" | wc -l)" -ne 0 ]]; then
        echo "Instance has no system-table under def_schema. Will create new one under sys schema:"
		create_new_table
else 
		echo "Instance have system-table under def_schema. Will move the old one to sys schema:"
		move_old_table
fi

echo "Starting Jethro LoadScheduler and maint"
service jethro start  $INSTANCE loadscheduler
service jethro start  $INSTANCE maint

echo "Restarting Jethro services"	
	service jethro stop  $INSTANCE 
	service jethro start  $INSTANCE 
echo "End of upgrade"  $INSTANCE
echo "You may run 'service jethro start $INSTANCE all' on all instance hosts."
exit 0