#!/bin/bash

ARGS=( "$@" )

LOG_OUT_FILE=/tmp/jd_env_`whoami`.out

SCRIPT_DIR=${JETHRO_HOME}/current/conf
> $LOG_OUT_FILE
chmod 644 $LOG_OUT_FILE

function java_env() {

  echo "INITIAL JAVA_HOME - $JAVA_HOME" >> $LOG_OUT_FILE

  #referenced from apache hive script
  if [ -e /usr/libexec/bigtop-detect-javahome ]; then
    . /usr/libexec/bigtop-detect-javahome
  elif [ -e /usr/lib/bigtop-utils/bigtop-detect-javahome ]; then
    . /usr/lib/bigtop-utils/bigtop-detect-javahome
  fi

  echo "AFTER BIGTOP JAVA_HOME - $JAVA_HOME" >> $LOG_OUT_FILE
  if [ -z "${JAVA_HOME}" ]; then
    echo "JAVA_HOME not set attempting to set it using java location" >> $LOG_OUT_FILE
    JAVA_HOME=$(readlink -nf $(which java) | xargs dirname | xargs dirname | xargs dirname)
    if [ ! -e "$JAVA_HOME" ]; then
      echo 'JAVA_HOME not exists'
      JAVA_HOME=""
    else
      if [ -e "$JAVA_HOME/jre" ]; then
        JAVA_HOME=$JAVA_HOME/jre
        JRE_HOME=$JAVA_HOME/jre
      fi
    fi
    echo "FINAL JAVA_HOME - $JAVA_HOME" >> $LOG_OUT_FILE
    export JAVA_HOME=$JAVA_HOME
  fi

  if [ -z "${JRE_HOME}" ]; then
    if [ -e "$JAVA_HOME/jre" ]; then
      JRE_HOME=$JAVA_HOME/jre
    elif [ -e "$JAVA_HOME/../jre" ]; then
      JRE_HOME=$JAVA_HOME/../jre
    else
      JAVA_HOME=$(readlink -nf $(which java) | xargs dirname | xargs dirname)
      JRE_HOME=$JAVA_HOME/jre
    fi

    LD_LIBRARY_PATH+=:${JRE_HOME}/lib/amd64/server/:${JRE_HOME}/lib/amd64/default/:
    
  fi

#Add java c++ libraries for jethro server
  LD_LIBRARY_PATH+=:/usr/lib:${JAVA_HOME}/lib/amd64/server/:${JAVA_HOME}/lib/amd64/default/:

#This looks redundant[Aviv]
  for so_dir in /usr/java/jdk*/jre/lib/amd64/server; do
    LD_LIBRARY_PATH+="$so_dir:"
  done


  echo "2 JAVA_HOME - $JAVA_HOME" >> $LOG_OUT_FILE
  echo "3 JRE_HOME  - $JRE_HOME" >> $LOG_OUT_FILE
  echo "================" >> $LOG_OUT_FILE
  echo "4 LD_LIBRARY_PATH- $LD_LIBRARY_PATH" >> $LOG_OUT_FILE
  echo "================" >> $LOG_OUT_FILE
}

# configure libSegFault 
RH_LIBSEGFAULT_PATH=/lib64/libSegFault.so
DEBIAN_LIBSEGFAULT_PATH=/lib/x86_64-linux-gnu/libSegFault.so
if [ -f  RH_LIBSEGFAULT_PATH ]; then
    LD_PRELOAD=$RH_LIBSEGFAULT_PATH
elif [ -f DEBIAN_LIBSEGFAULT_PATH ]; then 
    LD_PRELOAD=$DEBIAN_LIBSEGFAULT_PATH
else
    LD_PRELOAD=
fi

if [[ $# -lt 1 ]]; then
  SEGFAULT_OUTPUT_NAME="$JETHRO_HOME/instances/bt_jethro.txt"
elif [[ $# -lt 2 ]]; then
  JNAME=${ARGS[0]}
  SEGFAULT_OUTPUT_NAME="$JETHRO_HOME/instances/bt_jethro_$JNAME.txt"
else
  INSTANCE_NAME=${ARGS[0]}
  JNAME=${ARGS[1]}
  SEGFAULT_OUTPUT_NAME="$JETHRO_HOME/instances/$INSTANCE_NAME/log/bt_jethro_$JNAME.txt"
fi

LD_PRELOAD=${LD_PRELOAD}:$JETHRO_HOME/lib/libtcmalloc.so.4.3.0

LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:$JETHRO_HOME:$JETHRO_HOME/lib

# identify and set Java env (if needed)
java_env

#JD-5241
export JD_JAVA_ARCH=`java -XshowSettings:properties -version 2>&1 >/dev/null |  grep sun.arch.data.model | sed 's/.* = //'`


# attempt to identify hadoop distribution
HADOOP_DISTRO=GENERIC
hadoop version >/dev/null 2>&1
if [[ $? == 0 ]]; then
	#print hadoop version to out file
	echo "Hadoop version info:" >> $LOG_OUT_FILE
	echo `hadoop version`       >> $LOG_OUT_FILE

    HDP_CLASS_PATH=`hadoop classpath`
    if [[ ${HDP_CLASS_PATH} =~ .*CDH.* ]]; then
        HADOOP_DISTRO=CDH
    elif [[ ${HDP_CLASS_PATH} =~ .*/opt/mapr/.* ]]; then
        HADOOP_DISTRO=MapR
    elif [[ ${HDP_CLASS_PATH} =~ .*hdp.* ]]; then
        HADOOP_DISTRO=HDP
    fi
fi

echo "Distribution - $HADOOP_DISTRO" >> $LOG_OUT_FILE
echo "INITIAL HADOOP_HOME - $HADOOP_HOME" >> $LOG_OUT_FILE

# distro specific configuration
case $HADOOP_DISTRO in
CDH)
    export HADOOP_HOME=/opt/cloudera/parcels/CDH/lib/hadoop
    ;;
MapR) 
    if [[ -d /opt/mapr/hadoop/hadoop ]]; then 
      export HADOOP_HOME=/opt/mapr/hadoop/hadoop;
    else 
      export HADOOP_HOME=/home/hadoop
    fi
    export LD_LIBRARY_PATH+=:/opt/mapr/lib:
    ;;
HDP)
    export HADOOP_HOME=/usr/hdp/current/hadoop-client
    ;;
*)
    export HADOOP_HOME=/usr/lib/hadoop
    ;;
esac

echo "FINAL HADOOP_HOME - $HADOOP_HOME" >> $LOG_OUT_FILE

export LD_LIBRARY_PATH=${HADOOP_HOME}/lib/native:${LD_LIBRARY_PATH}


################This is the old way to define hadoop classpath, uncomment if you encoutered classpath issues - Begin
#for jar in ${HADOOP_HOME}/client/*.jar; do
#    CLASSPATH+="$jar:"
#    JD_HIVE_JDBC_CLASSPATH+="$jar:"
#done
#
#JD_HIVE_JDBC_CLASSPATH+="${HADOOP_CONF_DIR}:/etc/hadoop/conf"
#CLASSPATH+="${HADOOP_CONF_DIR}:/etc/hadoop/conf"
################This is the old way to define hadoop classpath, uncomment if you encoutered classpath issues  - End



################This is the new way to define hadoop classpath, comment if you encoutered classpath issues - Begin
#using --glob due to the following: https://issues.apache.org/jira/browse/HADOOP-10903
hadoop version >/dev/null 2>&1
if [[ $? == 0 ]]; then
CLASSPATH=`hadoop classpath --glob`:
JD_HIVE_JDBC_CLASSPATH=`hadoop classpath`:
fi
################This is the new way to define hadoop classpath, comment if you encoutered classpath issues - End

echo "CLASSPATH is - $CLASSPATH" >> $LOG_OUT_FILE



#---------Start setting hive drivers CLASSPATH----------------


#***************************************************#
#If you have encoutered an hive-jdbc missing warning you should uncomment the following line to point to the correct path
# of hive-jdbc-standalone.jar, make sure you 'yum install hive' first!!!
#Hint: the jar will probably be placed under the lib or the jdbc directory under the HIVE_HOME, 
# you can find the HIVE_HOME by running the following: "cat `which hive` | grep HIVE_HOME"

#JD_HIVE_JDBC_CLASSPATH+=HIVE_JDBC_FULL_PATH:
#***************************************************#



#used https://querysurge.zendesk.com/hc/en-us/articles/115001218863-Setting-Up-a-Hive-Connection-with-Kerberos-using-Apache-JDBC-Drivers-Windows-
# distro specific configuration
case $HADOOP_DISTRO in
GENERIC)
    #EMR:
    #Cloudera path for hive jdbc driver from url:https://www.cloudera.com/documentation/enterprise/latest/topics/cdh_ig_hive_jdbc_install.html#xd_583c10bfdbd326ba-3ca24a24-13d80143249--7eaf

    for jar in /usr/lib/hive/jdbc/hive-jdbc-*standalone.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

    for jar in /usr/lib/hive/lib/hive-jdbc-*standalone.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

    # Pulls all jars from hadoop client package and conf files from HADOOP_CONF_DIR
    for jar in /usr/lib/hadoop/hadoop-common.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

    for jar in /usr/lib/hadoop/hadoop-auth.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done
               
    for jar in /usr/lib/hadoop/lib/commons-configuration-*.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

    for jar in /usr/lib/hadoop-hdfs/lib/xercesImpl-*.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

    for jar in /usr/lib/hadoop-hdfs/lib/xml-apis-*.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done
    ;;
    
CDH)
#Cloudera parcels installation

    for jar in /opt/cloudera/parcels/CDH/jars/hive-jdbc-*standalone.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done
    
#Just in case, do deep search if the cloudera installation path
    for jar in `find /opt/cloudera/parcels/ -name "hive-jdbc*standalone.jar"`; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done
    
    ;;
HDP)
    #hortonworks path for hive jdbc driver from url:https://docs.hortonworks.com/HDPDocuments/HDP2/HDP-2.3.0/bk_dataintegration/content/hive-jdbc-odbc-drivers.html

    for jar in /usr/hdp/current/hive-client/jdbc/*.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done
    
#    for jar in /usr/hdp/current/hive-server2-hive2/jdbc/*.jar; do
#        JD_HIVE_JDBC_CLASSPATH+="$jar:"
#    done


    for jar in /usr/hdp/current/hadoop-client/client/hadoop-auth.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
	done

	for jar in /usr/hdp/current/hadoop-client/client/hadoop-common.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

	for jar in /usr/hdp/current/hadoop-client/lib/commons-configuration-*.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

	for jar in /usr/hdp/current/hadoop-client/client/xercesImpl.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

	for jar in /usr/hdp/current/hadoop-hdfs-client/lib/xml-apis-*.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

    ;;
MapR) 
    #http://doc.mapr.com/display/MapR/JDBC+Connections#JDBCConnections-UsingtheJDBCDriverinaMavenApplication
    for jar in /opt/mapr/hive/hive-1*/lib/hive-jdbc-*-standalone.jar; do
        JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done

#    for jar in /opt/mapr/hive/hive-2*/jdbc/hive-jdbc-*-standalone.jar; do
#        JD_HIVE_JDBC_CLASSPATH+="$jar:"
#    done    
    
    
    for jar in /opt/mapr/lib/hadoop-common-*.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done
    
    for jar in /opt/mapr/lib/hadoop-auth-*.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done
    
    for jar in /opt/mapr/lib/commons-configuration-*.jar; do
      JD_HIVE_JDBC_CLASSPATH+="$jar:"
    done
    ;;
esac

#---------End setting hive drivers CLASSPATH----------------

echo "JD_HIVE_JDBC_CLASSPATH is - $JD_HIVE_JDBC_CLASSPATH" >> $LOG_OUT_FILE

export JD_HIVE_JDBC_CLASSPATH="${JD_HIVE_JDBC_CLASSPATH}"
export CLASSPATH="${CLASSPATH}"
export HADOOP_CLASSPATH="${CLASSPATH}"
export LIBHDFS_OPTS="${LIBHDFS_OPTS} -Xmx4g"
export LD_PRELOAD

#needed for libhdfs to find hdfs-client.xml - not in use
#export LIBHDFS3_CONF=$JETHRO_HOME/conf/hdfs-client.xml

echo "================" >> $LOG_OUT_FILE
echo "LD_LIBRARY_PATH - $LD_LIBRARY_PATH" >> $LOG_OUT_FILE


#echo LOADED SUCCESSFULLY

